#
# spec file for sCxx environment...
#
Summary: A concurrent C++ environment providing Motif and Sockets libraries
Name: sCxx
%define version 1.0
Version: %{version}
%define GCC_VERSION 2.8.1
Release: 3
Copyright: EPFL-DI-LTI
Group: Development/Languages
Source: sCxx-1.0.tar.gz
Source1: sCxx-skel 
Source2: sCxx-libraries-makefile.tgz 
Patch0: sCxx-kernel-makefile.patch
URL: http://ltiwww.epfl.ch/sCxx/index.html
Distribution : LTI
Vendor: EPFL-DI-LTI
Packager: dbarth@besancon.net
Buildroot: /tmp/sCxx-root
Requires: binutils
Prefix: /usr/local

%description
This package provides a quick introduction to CONCURRENCY by
extending C++ with just three keywords that make the use of threads 
very easy.  With a limited number of lines of code, you can:
- create a MOTIF/LessTif Window, and read it from a thread, like with the
scanf function, but of course without blocking the other threads 
- establish a TCP CONNECTION between two computers (5 statements
including the handling of the socket daemon)
- create a simple HTTP server
This environment has allowed us to build CORBA orbs, debuggers,
computer aided teaching, and to validate applications formally.
People outside our laboratory, with no background in concurrency,
who have developed distributed applications (remote robot control, 
parallel molecule computations) with this approach are enthusiast.
More on this approach http://ltiwww.epfl.ch/sCxx.

%changelog

* Tue Sep 22 1998 David Barth <dbarth@besancon.net>

- some packages didn't make sense, they have been merged back.

* Sun Sep 13 1998 David Barth <dbarth@besancon.net>

- reworked the spec file provided by G.Duval <duval@di.epfl.ch>
- buildrooted
- subpackaged

%package gui
Requires: sCxx = %{version} 
Summary: sCxx GUI library
Group: Development/Libraries
Prefix: /usr/local
%description gui
This package contains the optional GUI library statically linked with a Motif-compatible library (depending on the build platform). This requires a Motif compatible development environment (mainly the include files).

%package examples
Requires: sCxx = %{version}, sCxx-gui = %{version}
Summary: sCxx examples
Group: Development/Languages
Prefix: /usr/local
%description examples
This package contains examples for sCxx. Most of the examples also require the GUI library.

%prep
%setup -n sCxx
# This simple one prevents the use of the INSTALL variable in the Makefile. Again to make it possible to build
# the whole package without root access (this way we are much sure that it will also be relocatable)
%patch0 -p1

# We must install our own makefiles for the libraries in order to build properly
# with a different default location. Also, the scripts we trying to create directories
# _before_ even compiling anything... Sorry no patch, just a quick hack

tar -zxf $RPM_SOURCE_DIR/sCxx-libraries-makefile.tgz
for i in libraries/CppUtils/Makefile libraries/GUI/Makefile libraries/Sockets/Makefile ; do
	mv $i $i.in
	sed "s#/tmp/gcc-root#${RPM_BUILD_ROOT}#" < $i.in > $i
done

# Taken from gcc-2.8.0.spec. It's probably unnecessary now, but I don't want to fiddle again with this until I get more horsepower ;-)
cp $RPM_SOURCE_DIR/sCxx-skel ./sCxx-skel.in
cat >> sCxx-skel.in <<EOF

/usr/local/sCxx/lib/gcc-lib/ARCH-redhat-linux/VERSION/crtbegin.o
/usr/local/sCxx/lib/gcc-lib/ARCH-redhat-linux/VERSION/crtbeginS.o
/usr/local/sCxx/lib/gcc-lib/ARCH-redhat-linux/VERSION/crtend.o
/usr/local/sCxx/lib/gcc-lib/ARCH-redhat-linux/VERSION/crtendS.o
EOF

sed "s/ARCH/${RPM_ARCH}/g" < sCxx-skel.in | \
	sed "s/VERSION/%{GCC_VERSION}/g" > sCxx-skel

%build
# make sure we don't mess with a previous build
rm -fr $RPM_BUILD_ROOT
cd gcc-%{GCC_VERSION}
./configure --prefix=/usr/local/sCxx ${RPM_ARCH}-redhat-linux    
make LANGUAGES="c c++"
make LANGUAGES="c c++" stage1
make LANGUAGES="c c++" CC="stage1/xgcc -Bstage1/" CFLAGS="-O" LDFLAGS="-s"
make LANGUAGES="c c++" stage2
make LANGUAGES="c c++" CC="stage2/xgcc -Bstage2/" CFLAGS="-O" LDFLAGS="-s"

# at this stage we are able to use the generated compiler without installing it 
# and use it to build the kernel
cd ../kernel
make LANGUAGES="c c++" CC="../gcc-%{GCC_VERSION}/stage2/xgcc -B../gcc-%{GCC_VERSION}/stage2/" CXX="../gcc-%{GCC_VERSION}/stage2/xgcc -B../gcc-%{GCC_VERSION}/stage2/" MACHINE_DIR="../gcc-%{GCC_VERSION}/stage2/"

# the libraries then, using the simple makefiles (they will probably have to be regenerated for the next version)
#
# WARNING: this creates _STATIC_ libs : GUI is huge !!!
# What about shared libraries ?
cd ../libraries
make

# the examples: I don't do anything at this point, assuming that the end user will install the package in a different location
# ie, not /usr/local/sCxx so that he will build with proper permissions 
cd ../examples
# do some configuration here


%install
cd gcc-%{GCC_VERSION}
rm -fr $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT/usr/local/sCxx/lib
# beware: use the same variables or make will try to rebuild the whole thing...
make prefix=$RPM_BUILD_ROOT/usr/local/sCxx LANGUAGES="c c++" CC="stage2/xgcc -Bstage2/" CXX="stage2/xgcc -Bstage2/" MACHINE_DIR="stage2/" install

# compress the huge info files
# perhaps they should be exported to another package : they are not installed (by install-info) and they seem
# to be equivalent to those included with gcc
gzip -n -9f $RPM_BUILD_ROOT/usr/local/sCxx/info/*.info*

# Should I also create a sc++ link ?
# Anyway, I don't make these links anymore as we probably don't want to invade the /usr/local/bin directory
#ln -sf gcc $RPM_BUILD_ROOT/usr/local/sCxx/bin/cc
#ln -sf cccp.1 $RPM_BUILD_ROOT/usr/local/sCxx/man/man1/cpp.1
#ln -sf ../usr/local/sCxx/lib/gcc-lib/${RPM_ARCH}-redhat-linux/${RPM_PACKAGE_VERSION}/cpp $RPM_BUILD_ROOT/usr/local/sCxx/lib/cpp

cd ../kernel
make PREFIX=$RPM_BUILD_ROOT/usr/local/sCxx LANGUAGES="c c++" CC="../gcc-%{GCC_VERSION}/stage2/xgcc -B../gcc-%{GCC_VERSION}/stage2/" CXX="../gcc-%{GCC_VERSION}/stage2/xgcc -B../gcc-%{GCC_VERSION}/stage2/" MACHINE_DIR="${RPM_ARCH}-redhat-linux" VERSION_DIR="%{GCC_VERSION}" install

cd ../libraries
make install
# manual tweeking : these files were left out of the makefile
# (un)intentionaly ?
for i in Sockets/libSocketsGlobal.h \
	 GUI/GUIMirrorTypes_i386-linux.h \
	 GUI/GUIMirrorTypeslinux.h ; do
	cp $i $RPM_BUILD_ROOT/usr/local/sCxx/include
done
# this needs a %ifos or something...
#  GUI/GUIMirrorTypes_alpha.h \
#  GUI/GUIMirrorTypes_sun4.h

cd ..
cp -R Documentation $RPM_BUILD_ROOT/usr/local/sCxx

cp -R examples $RPM_BUILD_ROOT/usr/local/sCxx

%clean
rm -rf $RPM_BUILD_ROOT


# Package files definitions (manually created)
%files
/usr/local/sCxx/lib/gcc-lib
/usr/local/sCxx/bin
/usr/local/sCxx/info
%docdir /usr/local/sCxx/info
/usr/local/sCxx/man
%docdir /usr/local/sCxx/man
/usr/local/sCxx/Documentation
%docdir /usr/local/sCxx/Documentation

/usr/local/sCxx/lib/libsc.a
/usr/local/sCxx/lib/libsc_g.a

/usr/local/sCxx/lib/libCppUtils.a
/usr/local/sCxx/lib/libSockets.a
/usr/local/sCxx/include/CharString.h
/usr/local/sCxx/include/DynamicArrayTemplate.h
/usr/local/sCxx/include/DynamicListTemplate.h
/usr/local/sCxx/include/Fifo.h
/usr/local/sCxx/include/FifoTemplate.h
/usr/local/sCxx/include/GetOpts.h
/usr/local/sCxx/include/ListTemplate.h
/usr/local/sCxx/include/Parser.h
/usr/local/sCxx/include/RandomIntGenerator.h
/usr/local/sCxx/include/SocketsGlobal.h
/usr/local/sCxx/include/TCPSocket.h
/usr/local/sCxx/include/UDPConn.h
/usr/local/sCxx/include/UDPSocket.h
/usr/local/sCxx/include/UnixGate.h
/usr/local/sCxx/include/UnixSocket.h
/usr/local/sCxx/include/libSocketsGlobal.h

# this one is set apart because it needs Motif
%files gui
/usr/local/sCxx/lib/libGUI.a
/usr/local/sCxx/include/ArrowButton.h
/usr/local/sCxx/include/BasicPushButton.h
/usr/local/sCxx/include/BulletinBoard.h
/usr/local/sCxx/include/CheckBox.h
/usr/local/sCxx/include/Command.h
/usr/local/sCxx/include/DrawingArea.h
/usr/local/sCxx/include/EHArrowButton.h
/usr/local/sCxx/include/EHBasicPushButton.h
/usr/local/sCxx/include/EHCommand.h
/usr/local/sCxx/include/EHDrawingArea.h
/usr/local/sCxx/include/EHFileSelection.h
/usr/local/sCxx/include/EHGUIList.h
/usr/local/sCxx/include/EHManualScrolledWindow.h
/usr/local/sCxx/include/EHScale.h
/usr/local/sCxx/include/EHSimpleButtonContainer.h
/usr/local/sCxx/include/EHSingleSelectionList.h
/usr/local/sCxx/include/EHTextField.h
/usr/local/sCxx/include/EHXTerm.h
/usr/local/sCxx/include/FileSelection.h
/usr/local/sCxx/include/Form.h
/usr/local/sCxx/include/GUI.h
/usr/local/sCxx/include/GUIGlobal.h
/usr/local/sCxx/include/GUIList.h
/usr/local/sCxx/include/GUIMirrorTypes.h
/usr/local/sCxx/include/GUIMirrorTypes_i386-linux.h
/usr/local/sCxx/include/GUIMirrorTypeslinux.h
/usr/local/sCxx/include/GUIMouse.h
/usr/local/sCxx/include/GUIRealizer.h
/usr/local/sCxx/include/GUITypes.h
/usr/local/sCxx/include/GWidget.h
/usr/local/sCxx/include/GWindow.h
/usr/local/sCxx/include/Label.h
/usr/local/sCxx/include/LabelPixmap.h
/usr/local/sCxx/include/MainWindow.h
/usr/local/sCxx/include/ManualScrolledWindow.h
/usr/local/sCxx/include/Menu.h
/usr/local/sCxx/include/MenuBar.h
/usr/local/sCxx/include/MenuButton.h
/usr/local/sCxx/include/MultipleSelectionList.h
/usr/local/sCxx/include/OptionMenu.h
/usr/local/sCxx/include/PanedWindow.h
/usr/local/sCxx/include/PopUpMenu.h
/usr/local/sCxx/include/PullDownMenu.h
/usr/local/sCxx/include/PushButton.h
/usr/local/sCxx/include/PushButtonPixmap.h
/usr/local/sCxx/include/RadioBox.h
/usr/local/sCxx/include/RowColumn.h
/usr/local/sCxx/include/Scale.h
/usr/local/sCxx/include/ScrolledWindow.h
/usr/local/sCxx/include/Separator.h
/usr/local/sCxx/include/SimpleButtonContainer.h
/usr/local/sCxx/include/SingleSelectionList.h
/usr/local/sCxx/include/Text.h
/usr/local/sCxx/include/TextField.h
/usr/local/sCxx/include/ToggleButton.h
/usr/local/sCxx/include/XTerm.h

%files examples
/usr/local/sCxx/examples
